"""
DEFAULT SETTINGS for create sharing snippets images.

Allowed settings:
SNIPPET_IMAGE_DEFAULT_SIZE,
SNIPPET_IMAGE_DEFAULT_BACKGROUND,
SNIPPET_IMAGE_DEFAULT_BACKGROUND_COLOR: as default is (0, 0, 0),
SNIPPET_IMAGE_DEFAULT_OVERLAY,
SNIPPET_IMAGE_DEFAULT_BRIGHTNESS: as default is 0.5,
SNIPPET_IMAGE_DEFAULT_FONT,
SNIPPET_IMAGE_DEFAULT_FONT_SIZE: as default is 64,
SNIPPET_IMAGE_DEFAULT_FONT_COLOR: as default is (255, 255, 255, 255),
SNIPPET_IMAGE_DEFAULT_PADDING: as defaults is 0.1,
"""
from django.conf import settings as django_settings


class LazySettings:
    def __init__(
            self,
            **defaults
    ):
        self.defaults = defaults

    def __getattr__(self, name):
        value = getattr(django_settings, name, self.defaults.get(name))

        return value


settings = LazySettings(
    SNIPPET_IMAGE_DEFAULT_BACKGROUND_COLOR=(0, 0, 0),
    SNIPPET_IMAGE_DEFAULT_BRIGHTNESS=0.5,
    SNIPPET_IMAGE_DEFAULT_FONT_SIZE=64,
    SNIPPET_IMAGE_DEFAULT_FONT_COLOR=(255, 255, 255, 255),
    SNIPPET_IMAGE_DEFAULT_PADDING=0.1,
)



from functools import wraps


def is_required(value):
    if value is None:
        raise ValueError('Value is required')


def is_not_required_validator(validator):
    @wraps(validator)
    def wrapper(value):
        if value:
            validator(value)

    return wrapper


@is_not_required_validator
def is_2d(value):
    measurement = list(value)
    if len(measurement) == 2 and not all(isinstance(part, int) for part in measurement):
        raise ValueError('Value must be 2d measurement(tuple(int, int))')


@is_not_required_validator
def is_rgb_color(value):
    measurement = list(value)
    if len(measurement) == 3 and not all(isinstance(part, int) for part in measurement):
        raise ValueError('Value must be RGB color(tuple(int, int, int))')


@is_not_required_validator
def is_rgba_color(value):
    measurement = list(value)
    if len(measurement) == 4 and not all(isinstance(part, int) for part in measurement):
        raise ValueError('Value must be RGB color(tuple(int, int, int))')




from django.db.models import ImageField
from uuid import uuid4
from snippet_image import create_snippet_image

from .attributes import Attributes


class BaseSnippetImageFieldMixin:
    method_prefix = 'get_snippet_image'
    snippet_type = 'default'
    kwargs = {}
    should_be_created_method = 'snippet_image_should_be_created'

    def extract_specific_kwargs(self, kwargs):
        return self.extract_kwargs(kwargs, Attributes)

    @staticmethod
    def extract_kwargs(kwargs, attributes):
        """
        Extract filled attributes from kwargs.

        :param kwargs: The dictionary from which attributes will be extracted.
        :type kwargs: dict
        :param attributes: The enumerate of attributes that will be retrieved from the dictionary.
        :type attributes: Enum

        :return: extracted kwargs.
        :rtype: dict
        """
        extracted_kwargs = {}
        for attribute in attributes:
            if kwargs.get(attribute.name):
                extracted_kwargs[attribute.name] = kwargs.pop(attribute.name)

        return extracted_kwargs

    def should_be_created(self, instance):
        method = getattr(instance, self.should_be_created_method, None)
        return method() if method else True

    @staticmethod
    def get_file_name():
        return '{}.jpeg'.format(str(uuid4()))

    def collect_data(self, instance):
        data = {
            attribute.name: self.get_attribute_value(instance, attribute) for attribute in Attributes
        }

        return data

    def get_attribute_value(self, instance, attribute):
        attribute_name = attribute.name
        value = self.get_attribute_from_kwargs(attribute_name)

        if not value:
            value = self.get_attribute_from_instance(instance, attribute_name)

        if not value:
            value = attribute.value.default

        self.validate_attribute(attribute, value)

        return value

    @staticmethod
    def validate_attribute(attribute, value):
        try:
            attribute.value.validate(value)
        except ValueError as error:
            raise ValueError('Validation error for attribute {}: {}'.format(attribute.name, str(error)))

    def get_attribute_from_kwargs(self, attribute_name):
        return self.kwargs.get(attribute_name)

    def get_attribute_from_instance(self, instance, attribute_name):
        method_name = '{}_{}'.format(self.method_prefix, attribute_name)
        method = getattr(instance, method_name, None)
        if method:
            value = method(self.snippet_type)
        else:
            value = None

        return value

    @staticmethod
    def create_snippet_image(data):
        return create_snippet_image(**data)

    def get_specific_deconstruct_kwargs(self):
        kwargs = {
            attribute.name: self.kwargs.get(attribute.name)
            for attribute in Attributes
            if not self.kwargs.get(attribute.name) is None
        }
        kwargs['snippet_type'] = self.snippet_type

        return kwargs


class SnippetImageField(BaseSnippetImageFieldMixin, ImageField):
    def __init__(
            self,
            snippet_type='default',
            **kwargs
    ):
        """

        :param snippet_type (str): To collect different data for different fields.
                                   For example value: 'facebook', 'twitter' and etc.
        :param kwargs: snippet_image.create_snippet_image and ImageField params.
            If create_snippet_image parameter was not set in the constructor,
            it will be taken from the method get_snippet_image_{param} of instance or default settings.
            create_snippet_image:
                :param font: Path to font file. Is required. For load font used PIL.ImageFont.
                :type font: str
                :param size: Size of snippet image. tuple(width, height).
                :type size: tuple(int, int)
                :param text: Text of snippet image. By default is an empty string.
                :type text: str
                :param background: Path to background image file.
                :type background: str
                :param background_color: Background color of snippet image. Used when background is None.
                :type background_color: tuple(int, int, int)
                :param overlay: Path to overlay image. if size is None, overlay size is used.
                                As an overlay, an image with a transparent background is used.
                :type overlay: str
                :param brightness: Brightness of background of snippet image. Value from 0 to 1.
                :type brightness: float
                :param font_color: Font color in RGBA. By default is (255, 255, 255, 255).
                :type font_color: tuple(int, int, int, int)
                :param font_size: Size of snippet image text. By default is 64.
                :type font_size: int
                :param padding: Text indents to the left and right of the snippet image.
                                Value from 0 to 1.
                                0 - 0% width;
                                1 - 100% width.
                :type padding: float
                :param center : Background image center for crop and resize image. tuple(x, y).
                                Defaults is center of background image.
                :type center: tuple(int, int)

            ImageField params:
                 :param verbose_name:
                 :param name:
                 :param width_field:
                 :param height_field:
                 :param upload_to:
                 :param storage:
        """
        self.snippet_type = snippet_type
        self.kwargs = self.extract_specific_kwargs(kwargs)
        kwargs['blank'] = True
        super().__init__(**kwargs)

    def pre_save(self, instance, add):
        file = getattr(instance, self.attname)

        if (not file or not file.file) and self.should_be_created(instance):
            file_name = self.get_file_name()
            data = self.collect_data(instance)
            image = self.create_snippet_image(data)
            file.save(file_name, image, save=False)
        elif not file._committed:
            file.save(file.name, file.file, save=False)

        return file

    def deconstruct(self):
        name, path, args, kwargs = super().deconstruct()

        specific_attributes_values = self.get_specific_deconstruct_kwargs()
        kwargs.update(specific_attributes_values)

        return name, path, args, kwargs



from enum import Enum

from .conf import settings
from .validators import (
    is_required,
    is_2d,
    is_rgb_color,
    is_rgba_color,
)


class Attribute:
    def __init__(
            self,
            default_setting=None,
            default_value=None,
            validators=None,
    ):
        self.default_setting = default_setting
        self.default_value = default_value
        self.validators = validators or []

    def validate(self, value=None):
        value = value or self.default
        for validator in self.validators:
            validator(value)

    @property
    def default(self):
        return self.default_setting and getattr(settings, self.default_setting) \
            if self.default_value is None else self.default_value


class Attributes(Enum):
    font = Attribute(
        default_setting='SNIPPET_IMAGE_DEFAULT_FONT',
        validators=[is_required, ],
    )
    size = Attribute(
        default_setting='SNIPPET_IMAGE_DEFAULT_SIZE',
        validators=[is_2d, ]
    )
    text = Attribute(default_value='')
    background = Attribute(default_setting='SNIPPET_IMAGE_DEFAULT_BACKGROUND')
    background_color = Attribute(
        default_setting='SNIPPET_IMAGE_DEFAULT_BACKGROUND_COLOR',
        validators=[is_rgb_color, ],
    )
    overlay = Attribute(default_setting='SNIPPET_IMAGE_DEFAULT_OVERLAY')
    brightness = Attribute(default_setting='SNIPPET_IMAGE_DEFAULT_BRIGHTNESS')
    font_color = Attribute(
        default_setting='SNIPPET_IMAGE_DEFAULT_FONT_COLOR',
        validators=[is_rgba_color, ]
    )
    font_size = Attribute(default_setting='SNIPPET_IMAGE_DEFAULT_FONT_SIZE')
    padding = Attribute(default_setting='SNIPPET_IMAGE_DEFAULT_PADDING')
    center = Attribute(validators=[is_2d, ])



# -*- coding: utf-8 -*-
from distutils.core import setup

packages = \
['django_snippet_image']

package_data = \
{'': ['*']}

install_requires = \
['django>=1.11,<3.0', 'snippet-image>=0.1.2,<0.2.0']

setup_kwargs = {
    'name': 'django-snippet-image',
    'version': '0.1.3',
    'description': 'Django field for create snippet image for sharing in social networks.',
    'long_description': '===========================\ndjango-snippet-image\n===========================\n\nThe python package provides a django field for automatic\ngeneration of images for sharing in social networks.\n\ndjango-snippet-image based on snippet-image_ package.\n\n.. _snippet-image: https://github.com/acrius/snippet-image\n\nInstallation\n-------------------------\n\n`\npip3 install django-snippet-image\n`\n\nExample\n-------------------------\n\nUse SnippetImageField:\n\n.. code-block:: python\n\n    from django.db.models import (\n        Model,\n        CharField,\n        ImageField,\n    )\n    from django_snippet_image import SnippetImageField\n\n\n    class Statuses:\n        DRAFT = \'draft\'\n        PUBLISH = \'publish\'\n\n        CHOICES = (\n            (DRAFT, \'Draft\'),\n            (PUBLISH, \'Publish\'),\n        )\n\n\n    class ExampleModel(Model):\n        text = CharField(\n            max_length=200,\n            verbose_name=\'Text for snippet image\',\n        )\n        background = ImageField(\n            verbose_name=\'Background for snippet image\',\n            blank=True,\n            null=True,\n        )\n        snippet_image_field = SnippetImageField(\n            verbose_name=\'Example snippet image field\',\n            null=True,\n        )\n        status = CharField(\n            max_length=20,\n            choices=Statuses.CHOICES,\n        )\n\n        # Methods for collect data for snippet image.\n\n        def get_snippet_image_text(self, snippet_type):\n            return self.text if snippet_type == \'default\' and self.text else \'\'\n\n        def get_snippet_image_background(self, snippet_type):\n            if snippet_type == \'default\' and self.background:\n                return self.background.path\n\n        def snippet_image_should_be_created(self):\n            return self.status == Statuses.PUBLISH\n\n        class Meta:\n            verbose_name = \'example object\'\n            verbose_name_plural = \'example objects\'\n\nAnd use in template:\n\n.. code-block:: html\n\n    <meta property="og:image" content="{{ instance.snippet_image_field.url }}" />\n\nRead more on home page_.\n\n.. _page: https://github.com/acrius/django-snippet-image\n',
    'author': 'acrius',
    'author_email': 'acrius@mail.ru',
    'url': 'https://github.com/acrius/django-snippet-image',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.5,<4.0',
}


setup(**setup_kwargs)
